import React__default, { useMemo } from 'react';
import { Progress, WarningPanel } from '@backstage/core-components';
import { useCortexApi } from '../../../utils/hooks.esm.js';
import { quantileRankSorted } from 'simple-statistics';
import { StatsCard } from '../../StatsCard/StatsCard.esm.js';
import { percentageToStatus } from '../../../styles/styles.esm.js';
import { isApplicableRuleOutcome } from '../../../utils/ScorecardRules.esm.js';

const EntityScorecardOverview = ({
  score
}) => {
  const {
    value: allScores,
    loading,
    error
  } = useCortexApi(
    async (cortexApi) => {
      return score !== void 0 ? await cortexApi.getScorecardScores(score.scorecard.id) : void 0;
    },
    [score]
  );
  const sortedScores = useMemo(
    () => {
      var _a;
      return (_a = allScores == null ? void 0 : allScores.map((currScore) => currScore.scorePercentage)) == null ? void 0 : _a.sort((a, b) => a - b);
    },
    [allScores]
  );
  if (loading || score === void 0 || sortedScores === void 0) {
    return /* @__PURE__ */ React__default.createElement(Progress, null);
  }
  if (error) {
    return /* @__PURE__ */ React__default.createElement(WarningPanel, { severity: "error", title: "Failed to load scorecard." }, error.message);
  }
  if (sortedScores.length === 0) {
    return /* @__PURE__ */ React__default.createElement(
      WarningPanel,
      {
        severity: "error",
        title: "Error getting stats for service score."
      }
    );
  }
  const rank = [...new Set(sortedScores)].reverse().indexOf(score.score.scorePercentage) + 1;
  const percentileRank = quantileRankSorted(
    sortedScores,
    score.score.scorePercentage
  );
  const applicableRules = score.evaluation.rules.filter(
    (rule) => isApplicableRuleOutcome(rule)
  );
  const numFailingRules = applicableRules.filter(
    (rule) => rule.score === 0
  ).length;
  return /* @__PURE__ */ React__default.createElement(
    StatsCard,
    {
      stats: [
        {
          label: "Rank",
          status: percentageToStatus(percentileRank),
          value: rank,
          type: "NTH"
        },
        {
          label: "Percentile",
          status: percentageToStatus(percentileRank),
          value: percentileRank * 100,
          type: "NTH",
          percentage: true
        },
        {
          label: "Failing Rules",
          status: percentageToStatus(
            1 - numFailingRules / Math.max(applicableRules.length, 1)
          ),
          value: numFailingRules
        }
      ]
    }
  );
};

export { EntityScorecardOverview };
//# sourceMappingURL=EntityScorecardOverview.esm.js.map
