import React__default, { useMemo } from 'react';
import { Progress, WarningPanel, InfoCard } from '@backstage/core-components';
import { makeStyles, Grid, FormControl, TextField, InputAdornment, Table, TableBody, Typography } from '@material-ui/core';
import { EntityScorecardsCardRow } from './EntityScorecardsCardRow.esm.js';
import { SortDropdown } from '../Common/SortDropdown.esm.js';
import { useInput, usePartialScorecardCompareFn, useDropdown } from '../../utils/hooks.esm.js';
import SearchIcon from '@material-ui/icons/Search';
import { searchItems } from '../../utils/SearchUtils.esm.js';
import { isNil, isEmpty } from 'lodash';

const useStyles = makeStyles((theme) => ({
  table: {
    "&:nth-of-type(odd)": {
      backgroundColor: `${theme.palette.background.paper}!important`
    },
    "&:nth-of-type(even)": {
      backgroundColor: `${theme.palette.background.paper}!important`
    }
  }
}));
const defaultSortMethods = {
  "Name \u2191": (a, b) => a.scorecard.name.localeCompare(b.scorecard.name),
  "Name \u2193": (a, b) => b.scorecard.name.localeCompare(a.scorecard.name),
  "Score \u2191": (a, b) => a.score.scorePercentage > b.score.scorePercentage ? 1 : -1,
  "Score \u2193": (a, b) => b.score.scorePercentage > a.score.scorePercentage ? 1 : -1
};
const EntityScorecardsCard = ({
  title,
  scores,
  selectedScorecardId,
  onSelect
}) => {
  const classes = useStyles();
  const [searchQuery, setSearchQuery] = useInput();
  const {
    compareFn: scorecardCompareFn,
    loading: loadingScorecardCompareFn,
    error: scorecardCompareFnError
  } = usePartialScorecardCompareFn();
  const customScoresSortMethods = useMemo(() => {
    if (isNil(scorecardCompareFn)) {
      return {};
    }
    return {
      "Custom \u2191": (a, b) => scorecardCompareFn(a.scorecard, b.scorecard),
      "Custom \u2193": (a, b) => -1 * scorecardCompareFn(a.scorecard, b.scorecard)
    };
  }, [scorecardCompareFn]);
  const scorecardScoresSortMethods = useMemo(() => {
    return {
      ...customScoresSortMethods,
      ...defaultSortMethods
    };
  }, [customScoresSortMethods]);
  const [sortBy, setSortBy] = useDropdown(
    isEmpty(customScoresSortMethods) ? "Name \u2191" : "Custom \u2191",
    [isEmpty(customScoresSortMethods)]
  );
  const scoresToDisplay = useMemo(() => {
    const scoresToDisplay2 = searchItems(
      scores,
      ["scorecard.name", "scorecard.description"],
      searchQuery
    );
    if (sortBy) {
      scoresToDisplay2.sort(scorecardScoresSortMethods[sortBy]);
    }
    return scoresToDisplay2;
  }, [scores, searchQuery, sortBy, scorecardScoresSortMethods]);
  if (loadingScorecardCompareFn) {
    return /* @__PURE__ */ React__default.createElement(Progress, null);
  }
  if (scorecardCompareFnError) {
    return /* @__PURE__ */ React__default.createElement(WarningPanel, { severity: "error", title: "Could not load Scorecards." }, scorecardCompareFnError.message);
  }
  return /* @__PURE__ */ React__default.createElement(InfoCard, { title }, /* @__PURE__ */ React__default.createElement(Grid, { container: true, direction: "column" }, /* @__PURE__ */ React__default.createElement(
    Grid,
    {
      container: true,
      direction: "row",
      lg: 12,
      style: { marginBottom: "20px" }
    },
    /* @__PURE__ */ React__default.createElement(Grid, { item: true, lg: 9 }, /* @__PURE__ */ React__default.createElement(FormControl, { fullWidth: true }, /* @__PURE__ */ React__default.createElement(
      TextField,
      {
        variant: "standard",
        placeholder: "Search by name, description, or filters",
        value: searchQuery,
        onChange: setSearchQuery,
        InputProps: {
          startAdornment: /* @__PURE__ */ React__default.createElement(InputAdornment, { position: "start" }, /* @__PURE__ */ React__default.createElement(SearchIcon, null))
        }
      }
    ))),
    /* @__PURE__ */ React__default.createElement(Grid, { item: true, lg: 3 }, /* @__PURE__ */ React__default.createElement(
      SortDropdown,
      {
        selected: sortBy,
        items: Object.keys(scorecardScoresSortMethods),
        select: setSortBy
      }
    ))
  ), /* @__PURE__ */ React__default.createElement(Table, { className: classes.table }, /* @__PURE__ */ React__default.createElement(TableBody, null, isEmpty(scoresToDisplay) && !isNil(searchQuery) && /* @__PURE__ */ React__default.createElement(Typography, { variant: "subtitle1" }, "No Scorecards matching search query"), scoresToDisplay.map((score) => /* @__PURE__ */ React__default.createElement(
    EntityScorecardsCardRow,
    {
      key: `EntityScorecardsCardRow-${score.scorecard.id}`,
      score,
      onSelect: () => onSelect(score.scorecard.id),
      selected: selectedScorecardId === score.scorecard.id
    }
  ))))));
};

export { EntityScorecardsCard };
//# sourceMappingURL=EntityScorecardsCard.esm.js.map
