import { isEmpty, isNil } from 'lodash';
import { RELATION_OWNED_BY, parseEntityRef, RELATION_PART_OF } from '@backstage/catalog-model';
import { getEntityRelations, humanizeEntityRef } from '@backstage/plugin-catalog-react';
import { defaultGroupRefContext, defaultSystemRefContext } from '../../../../utils/ComponentUtils.esm.js';
import { combinePredicates, stringifyAnyEntityRef } from '../../../../utils/types.esm.js';

var FilterDefinitionName = /* @__PURE__ */ ((FilterDefinitionName2) => {
  FilterDefinitionName2["FailingRules"] = "Failing rules";
  FilterDefinitionName2["PassingRules"] = "Passing rules";
  FilterDefinitionName2["EmailOwners"] = "Email owners";
  FilterDefinitionName2["ShowActionItemsFor"] = "myEntities";
  return FilterDefinitionName2;
})(FilterDefinitionName || {});
const groupAndSystemFilters = [
  {
    name: "Groups",
    groupProperty: (entity) => getEntityRelations(entity, RELATION_OWNED_BY, {
      kind: "group"
    }).map(
      (entityRef) => stringifyAnyEntityRef(entityRef, { defaultKind: "group" })
    ),
    formatProperty: (groupRef) => humanizeEntityRef(parseEntityRef(groupRef), defaultGroupRefContext)
  },
  {
    name: "Systems",
    groupProperty: (entity) => getEntityRelations(entity, RELATION_PART_OF, {
      kind: "system"
    }).map(
      (entityRef) => stringifyAnyEntityRef(entityRef, { defaultKind: "system" })
    ),
    formatProperty: (groupRef) => humanizeEntityRef(parseEntityRef(groupRef), defaultSystemRefContext)
  }
];
const toPredicateFilters = (filters, name) => {
  return Object.keys(filters).filter((key) => key.includes(name)).reduce((acc, key) => {
    const newKey = key.replace(name, "");
    acc[newKey] = filters[key];
    return acc;
  }, {});
};
const getPredicateFilterFromFilters = (filters, oneOf, filterDefinitions) => {
  const allFilters = filterDefinitions.reduce((acc, filter) => {
    var _a;
    const predicateFilters = toPredicateFilters(filters, filter.name);
    const filterOneOf = (_a = oneOf[filter.name]) != null ? _a : true;
    acc[filter.name] = (componentRef) => {
      if (isEmpty(predicateFilters)) {
        return true;
      }
      const results = Object.keys(predicateFilters).filter((id) => predicateFilters[id] && !isNil(filter.filters[id])).map(
        (id) => filter.generatePredicate(filter.filters[id].value)(componentRef)
      );
      return isEmpty(results) || (filterOneOf ? results.some(Boolean) : results.every(Boolean));
    };
    return acc;
  }, {});
  return combinePredicates(Object.values(allFilters));
};
const oneOfPrefix = "oneOf__";
const toQueryParams = (filters, oneOf, filterDefinitions) => {
  const result = Object.entries(filters).reduce(
    (acc, [key, value]) => {
      if (!value) {
        return acc;
      }
      const filter = filterDefinitions.find(
        (filter2) => key.includes(filter2.name)
      );
      if (!filter) {
        return acc;
      }
      const filterEntry = key.replace(filter.name, "");
      if (!acc[filter.name]) {
        acc[filter.name] = [];
      }
      acc[filter.name].push(filterEntry);
      return acc;
    },
    {}
  );
  const oneOfResult = Object.entries(oneOf).reduce(
    (acc, [key, value]) => {
      if (!value) {
        acc[`${oneOfPrefix}${key}`] = false;
      }
      return acc;
    },
    {}
  );
  return {
    ...oneOfResult,
    ...result
  };
};
const useFiltersFromQueryParams = (queryParams) => {
  const searchParams = new URLSearchParams(queryParams);
  const filters = {};
  const oneOf = {};
  searchParams.forEach((value, key) => {
    if (key.startsWith(oneOfPrefix)) {
      oneOf[key.replace(oneOfPrefix, "")] = value === "true";
    } else {
      const filterName = Object.values(FilterDefinitionName).find(
        (filterName2) => key.includes(filterName2)
      );
      filters[`${filterName}${value}`] = true;
    }
  });
  return { filters, oneOf };
};
const createRulePredicate = (rule, actionItems, pass) => {
  return (componentRef) => {
    const passed = !actionItems.some(
      (actionItem) => actionItem.componentRef === componentRef && actionItem.rule.expression === rule
    );
    return passed === pass;
  };
};
const myEntitiesValue = "my entities";
const showActionItemForFilters = {
  // Defined as "true" to make it look nicer in the URL (url?myEntities=true instead of url?myEntities=myEntities)
  true: {
    display: "My entities",
    id: myEntitiesValue,
    value: myEntitiesValue
  }
};
const getFilterDefinitions = ({
  actionItems,
  entitiesByTag,
  ownerOptionsMap,
  ruleFilterDefinitions,
  userEntities
}) => {
  return [
    {
      name: "myEntities" /* ShowActionItemsFor */,
      displayName: "Show action items for",
      filters: showActionItemForFilters,
      oneOfDisabled: true,
      generatePredicate: (restriction) => {
        return (componentRef) => {
          return restriction !== myEntitiesValue || userEntities.includes(componentRef);
        };
      }
    },
    {
      name: "Failing rules" /* FailingRules */,
      filters: ruleFilterDefinitions,
      generatePredicate: (failingRule) => createRulePredicate(failingRule, actionItems, false)
    },
    {
      name: "Passing rules" /* PassingRules */,
      filters: ruleFilterDefinitions,
      generatePredicate: (passingRule) => createRulePredicate(passingRule, actionItems, true)
    },
    {
      name: "Email owners" /* EmailOwners */,
      filters: ownerOptionsMap,
      generatePredicate: (emailOwner) => {
        return (componentRef) => {
          const entity = entitiesByTag[componentRef];
          if (!entity) {
            return false;
          }
          return entity.serviceOwnerEmails.some(
            (email) => email.email === emailOwner
          );
        };
      }
    }
  ];
};

export { FilterDefinitionName, getFilterDefinitions, getPredicateFilterFromFilters, groupAndSystemFilters, toPredicateFilters, toQueryParams, useFiltersFromQueryParams };
//# sourceMappingURL=InitiativeFilterDialogUtils.esm.js.map
