import React__default, { useMemo } from 'react';
import { ServiceNameAndRulesColumn } from './ServiceNameAndRulesColumn.esm.js';
import { InfoCard, EmptyState, Table } from '@backstage/core-components';
import { Typography, Box, ThemeProvider } from '@material-ui/core';
import { defaultComponentRefContext, entityComponentRef } from '../../../../utils/ComponentUtils.esm.js';
import { humanizeAnyEntityRef } from '../../../../utils/types.esm.js';
import { LinearProgressWithLabel } from '../../../Common/LinearProgressWithLabel.esm.js';
import { percentify } from '../../../../utils/NumberUtils.esm.js';
import { useInitiativeFailingTabStyle, failingTabTheme } from './InitiativeFailingTabConfig.esm.js';
import { groupBy, size } from 'lodash';
import useScorecardServiceRuleRowStyle from '../../../Scorecards/ScorecardsServicePage/useScorecardServiceRuleRowStyles.esm.js';
import useLinearProgressWithLabelStyles from '../../../Common/useLinearProgressWithLabelStyles.esm.js';

const InitiativeFailingTab = ({
  actionItems,
  defaultPageSize = 15,
  entitiesByTag,
  numRules = 2,
  scorecardId
}) => {
  const classes = useInitiativeFailingTabStyle();
  const failingComponents = useMemo(
    () => groupBy(actionItems, (actionItem) => actionItem.componentRef),
    [actionItems]
  );
  const data = useMemo(() => {
    return Object.keys(failingComponents).map((componentRef) => {
      const tag = humanizeAnyEntityRef(
        componentRef,
        defaultComponentRefContext
      );
      const { name, description } = entitiesByTag[componentRef];
      const serviceActionItems = failingComponents[componentRef];
      return {
        actionItems: serviceActionItems,
        componentRef,
        description,
        id: `${componentRef}-Failing`,
        // mui-table uses id internally
        name,
        tag,
        score: (numRules - serviceActionItems.length) / numRules
      };
    }).sort((left, right) => left.actionItems.length - right.actionItems.length).sort((left, right) => left.tag.localeCompare(right.tag));
  }, [entitiesByTag, failingComponents, numRules]);
  const showPagination = useMemo(() => {
    return size(failingComponents) > defaultPageSize;
  }, [failingComponents, defaultPageSize]);
  const scorecardServiceRuleRowClasses = useScorecardServiceRuleRowStyle();
  const linearProgressWithLabelClasses = useLinearProgressWithLabelStyles();
  const columns = useMemo(
    () => [
      {
        field: "all",
        title: "Service name",
        width: "60%",
        render: (data2) => {
          return /* @__PURE__ */ React__default.createElement(
            ServiceNameAndRulesColumn,
            {
              ...data2,
              componentRef: entityComponentRef(
                entitiesByTag,
                data2.componentRef
              ),
              scorecardServiceRuleRowClasses,
              scorecardId
            }
          );
        },
        customSort: (data1, data2) => {
          return data2.name.localeCompare(data1.name);
        },
        customFilterAndSearch: (filter, rowData) => {
          var _a, _b, _c, _d;
          return (_d = (_c = (_a = rowData.name) == null ? void 0 : _a.includes(filter)) != null ? _c : (_b = rowData.tag) == null ? void 0 : _b.includes(filter)) != null ? _d : false;
        },
        sorting: true
      },
      {
        sorting: true,
        field: "componentRef",
        title: "Tag",
        width: "25%",
        customSort: (data1, data2) => {
          return data2.tag.localeCompare(data1.tag);
        },
        customFilterAndSearch: (filter, rowData) => {
          var _a;
          return (_a = rowData.componentRef.toLocaleLowerCase().includes(filter.toLocaleLowerCase())) != null ? _a : false;
        },
        render: ({ componentRef }) => /* @__PURE__ */ React__default.createElement(Typography, { variant: "subtitle2", className: classes.tag }, componentRef)
      },
      {
        field: "score",
        sorting: true,
        title: "Score",
        width: "15%",
        render: (data2) => {
          var _a, _b;
          const actionItemsLength = (_b = (_a = data2.actionItems) == null ? void 0 : _a.length) != null ? _b : 0;
          return /* @__PURE__ */ React__default.createElement(Box, { sx: { width: 120 } }, /* @__PURE__ */ React__default.createElement(
            LinearProgressWithLabel,
            {
              classes: linearProgressWithLabelClasses,
              value: percentify(data2.score),
              label: `${numRules - actionItemsLength} / ${numRules}`
            }
          ));
        },
        customSort: (data1, data2) => {
          return data2.score - data1.score;
        }
      }
    ],
    [
      classes.tag,
      entitiesByTag,
      linearProgressWithLabelClasses,
      numRules,
      scorecardId,
      scorecardServiceRuleRowClasses
    ]
  );
  if (data.length === 0) {
    return /* @__PURE__ */ React__default.createElement(InfoCard, { title: "Failing" }, /* @__PURE__ */ React__default.createElement(EmptyState, { missing: "data", title: "No failing services." }));
  }
  return /* @__PURE__ */ React__default.createElement(Box, { marginBottom: 3 }, /* @__PURE__ */ React__default.createElement(ThemeProvider, { theme: failingTabTheme }, /* @__PURE__ */ React__default.createElement(
    Table,
    {
      columns,
      data,
      options: {
        pageSize: defaultPageSize,
        pageSizeOptions: [
          defaultPageSize,
          defaultPageSize * 2,
          defaultPageSize * 4
        ],
        paging: showPagination
      },
      title: "Failing"
    }
  )));
};

export { InitiativeFailingTab };
//# sourceMappingURL=InitiativeFailingTab.esm.js.map
