import React__default, { useMemo } from 'react';
import { cortexApiRef } from '../../../api/CortexClient.esm.js';
import { useAsync } from 'react-use';
import { Progress, WarningPanel, EmptyState, Content, ContentHeader, ItemCardGrid } from '@backstage/core-components';
import { useApi } from '@backstage/core-plugin-api';
import { InitiativeCard } from '../InitiativeCard/InitiativeCard.esm.js';
import { Grid, FormControl, TextField, InputAdornment } from '@material-ui/core';
import SearchIcon from '@material-ui/icons/Search';
import { isNil, isEmpty, isUndefined, capitalize } from 'lodash';
import { useInput, useDropdown, useInitiativesCustomName } from '../../../utils/hooks.esm.js';
import { hasText } from '../../../utils/SearchUtils.esm.js';
import { SortDropdown } from '../../Common/SortDropdown.esm.js';
import moment from 'moment';

const defaultSortMethods = {
  "Name \u2191": (a, b) => a.name.localeCompare(b.name),
  "Name \u2193": (a, b) => b.name.localeCompare(a.name),
  "Due date \u2191": (a, b) => moment(a.targetDate).diff(b.targetDate),
  "Due date \u2193": (a, b) => moment(b.targetDate).diff(a.targetDate)
};
const InitiativesList = () => {
  const cortexApi = useApi(cortexApiRef);
  const [searchQuery, setSearchQuery] = useInput();
  const {
    value: initiatives,
    loading,
    error
  } = useAsync(async () => {
    return await cortexApi.getInitiatives();
  }, []);
  const [sortBy, setSortBy] = useDropdown("Name \u2191");
  const initiativesToDisplay = useMemo(() => {
    const initiativesToDisplay2 = initiatives == null ? void 0 : initiatives.filter((initiative) => {
      if (isNil(searchQuery) || isEmpty(searchQuery)) {
        return true;
      }
      return hasText(initiative, "name", searchQuery) || hasText(initiative, "description", searchQuery) || hasText(initiative, "scorecard.name", searchQuery) || hasText(initiative, "scorecard.description", searchQuery);
    });
    if (sortBy) {
      initiativesToDisplay2 == null ? void 0 : initiativesToDisplay2.sort(defaultSortMethods[sortBy]);
    }
    return initiativesToDisplay2;
  }, [initiatives, searchQuery, sortBy]);
  const { plural: initiativesName } = useInitiativesCustomName();
  if (loading || isUndefined(initiativesToDisplay)) {
    return /* @__PURE__ */ React__default.createElement(Progress, null);
  }
  if (error) {
    return /* @__PURE__ */ React__default.createElement(WarningPanel, { severity: "error", title: `Could not load ${initiativesName}.` }, error.message);
  }
  if (!(initiatives == null ? void 0 : initiatives.length)) {
    return /* @__PURE__ */ React__default.createElement(
      EmptyState,
      {
        missing: "info",
        title: `No ${initiativesName} to display`,
        description: `You haven't added any ${initiativesName} yet.`
      }
    );
  }
  return /* @__PURE__ */ React__default.createElement(Content, null, /* @__PURE__ */ React__default.createElement(ContentHeader, { title: capitalize(initiativesName) }), /* @__PURE__ */ React__default.createElement(Grid, { container: true, direction: "column" }, /* @__PURE__ */ React__default.createElement(
    Grid,
    {
      container: true,
      direction: "row",
      lg: 12,
      style: { marginBottom: "20px" }
    },
    /* @__PURE__ */ React__default.createElement(Grid, { item: true, lg: 10 }, /* @__PURE__ */ React__default.createElement(FormControl, { fullWidth: true }, /* @__PURE__ */ React__default.createElement(
      TextField,
      {
        variant: "standard",
        placeholder: "Search by name, description, or filters",
        value: searchQuery,
        onChange: setSearchQuery,
        InputProps: {
          startAdornment: /* @__PURE__ */ React__default.createElement(InputAdornment, { position: "start" }, /* @__PURE__ */ React__default.createElement(SearchIcon, null))
        }
      }
    ))),
    /* @__PURE__ */ React__default.createElement(Grid, { item: true, lg: 2 }, /* @__PURE__ */ React__default.createElement(
      SortDropdown,
      {
        selected: sortBy,
        items: Object.keys(defaultSortMethods),
        select: setSortBy
      }
    ))
  ), /* @__PURE__ */ React__default.createElement(Grid, { item: true, lg: 12 }, isEmpty(initiativesToDisplay) && !isNil(searchQuery) && /* @__PURE__ */ React__default.createElement(
    EmptyState,
    {
      title: `No ${initiativesName} matching search query`,
      missing: "data"
    }
  ), /* @__PURE__ */ React__default.createElement(ItemCardGrid, null, initiativesToDisplay.map((initiative) => /* @__PURE__ */ React__default.createElement(
    InitiativeCard,
    {
      key: `InitiativeCard-${initiative.id}`,
      initiative
    }
  ))))));
};

export { InitiativesList };
//# sourceMappingURL=InitiativesList.esm.js.map
