import React__default, { useMemo, useCallback } from 'react';
import { EmptyState } from '@backstage/core-components';
import { useCortexBirdseye, Breadcrumb, BirdsEyeReportTable } from '@cortexapps/birdseye';
import { mapKeys, mapValues } from 'lodash';
import { HeatmapSettings } from './HeatmapSettings.esm.js';
import { useColorCellStyles } from './colorClasses.esm.js';
import { Grid } from '@material-ui/core';
import { useRouteRef } from '@backstage/core-plugin-api';
import { scorecardServiceDetailsRouteRef } from '../../../routes.esm.js';
import { entityComponentRef, defaultComponentRefContext } from '../../../utils/ComponentUtils.esm.js';
import { parseEntityRef } from '@backstage/catalog-model';
import { useTableStyles, convertToBirdsEyeScorecard, convertToBirdsEyeScores, convertToBirdsEyeDomainHierarchy, convertToBirdsEyeTeamHierarchy, getCellColorBackground, getScoreColorClassName, BirdsEyeAnchorAdapter } from './HeatmapUtils.esm.js';

const HeatmapTable = ({
  allDomains,
  allTeams,
  catalog,
  domainAncestryMap,
  domainHierarchy,
  entitiesByTag,
  filters,
  ladder,
  scorecard,
  scores,
  setFilters,
  teamsByEntity,
  teamHierarchy
}) => {
  const colorStyles = useColorCellStyles();
  const styles = useTableStyles();
  const {
    mappedDomainAncestryMap,
    mappedDomainHierarchies,
    mappedScorecard,
    mappedScores,
    mappedTeamHierarchies
  } = useMemo(() => {
    const mappedScorecard2 = convertToBirdsEyeScorecard(scorecard, ladder);
    const mappedScores2 = convertToBirdsEyeScores(scores, catalog);
    const mappedDomainHierarchies2 = convertToBirdsEyeDomainHierarchy(domainHierarchy);
    const mappedTeamHierarchies2 = convertToBirdsEyeTeamHierarchy(teamHierarchy);
    const mappedDomainAncestryMapKeys = mapKeys(
      domainAncestryMap,
      (_, key) => key.toString()
    );
    const mappedDomainAncestryMap2 = mapValues(
      mappedDomainAncestryMapKeys,
      (values) => values.map((value) => value.toString())
    );
    return {
      mappedScorecard: mappedScorecard2,
      mappedScores: mappedScores2,
      mappedDomainHierarchies: mappedDomainHierarchies2,
      mappedDomainAncestryMap: mappedDomainAncestryMap2,
      mappedTeamHierarchies: mappedTeamHierarchies2
    };
  }, [
    catalog,
    domainAncestryMap,
    domainHierarchy,
    ladder,
    scorecard,
    scores,
    teamHierarchy
  ]);
  const {
    tableData,
    setDataFilters,
    groupByOptions,
    setReportType,
    filtersConfig,
    setHideTeamsWithoutEntities,
    setGroupBy,
    setUseHierarchy,
    showHierarchy,
    groupBy,
    shouldShowReportType,
    breadcrumbItems,
    onBreadcrumbClick
  } = useCortexBirdseye({
    allDomains,
    allTeams,
    domainAncestryMap: mappedDomainAncestryMap,
    domainHierarchy: mappedDomainHierarchies,
    filters,
    scorecard: mappedScorecard,
    scores: mappedScores,
    setFilters,
    teamHierarchy: mappedTeamHierarchies,
    teamsByEntity
  });
  const scorecardServiceDetailsRef = useRouteRef(
    scorecardServiceDetailsRouteRef
  );
  const cellColorClassNameHandler = useCallback(
    (points) => {
      return `${colorStyles.root} ${colorStyles[getCellColorBackground(points)]}`;
    },
    [colorStyles]
  );
  const scoreColorClassNameHandler = useCallback(
    (points) => {
      return `${colorStyles.root} ${colorStyles[getScoreColorClassName(points)]}`;
    },
    [colorStyles]
  );
  const scorecardEntityUrlHandler = useCallback(
    (scorecardEntity) => {
      const codeTag = scorecardEntity.entityTag;
      const componentRef = entityComponentRef(entitiesByTag, codeTag);
      const entityName = parseEntityRef(
        componentRef,
        defaultComponentRefContext
      );
      const entityUrl = scorecardServiceDetailsRef({
        scorecardId: `${mappedScorecard.id}`,
        ...entityName
      });
      return entityUrl;
    },
    [entitiesByTag, mappedScorecard.id, scorecardServiceDetailsRef]
  );
  return /* @__PURE__ */ React__default.createElement(Grid, { container: true, direction: "column" }, /* @__PURE__ */ React__default.createElement(Grid, { item: true, style: { marginTop: "20px" } }, /* @__PURE__ */ React__default.createElement(
    Breadcrumb,
    {
      className: styles.breadcrumb,
      items: breadcrumbItems,
      onClick: onBreadcrumbClick
    }
  )), /* @__PURE__ */ React__default.createElement(Grid, { item: true }, /* @__PURE__ */ React__default.createElement(
    HeatmapSettings,
    {
      filters,
      groupByOptions,
      setGroupBy,
      filtersConfig,
      groupBy,
      setDataFilters,
      setHideTeamsWithoutEntities,
      setReportType,
      setUseHierarchy,
      shouldShowReportType,
      showHierarchy
    }
  )), /* @__PURE__ */ React__default.createElement(Grid, { item: true }, /* @__PURE__ */ React__default.createElement(
    BirdsEyeReportTable,
    {
      ...tableData,
      className: styles.table,
      components: {
        anchor: BirdsEyeAnchorAdapter
      },
      emptyResultDisplay: /* @__PURE__ */ React__default.createElement(EmptyState, { title: "No data", missing: "data" }),
      filters,
      getCellColorClassName: cellColorClassNameHandler,
      getScoreColorClassName: scoreColorClassNameHandler,
      getScorecardEntityUrl: scorecardEntityUrlHandler,
      setFilters
    }
  )));
};

export { HeatmapTable };
//# sourceMappingURL=HeatmapTable.esm.js.map
