import React__default, { useMemo } from 'react';
import { makeStyles, Dialog, DialogContent, DialogActions, Button } from '@material-ui/core';
import { ruleName } from '../../../../api/types.esm.js';
import { FilterProvider, useFilter } from '../../../FilterCard/useFilter.esm.js';
import { mapValues } from 'lodash';
import { mapByString } from '../../../../utils/collections.esm.js';
import { useFilters } from '../../../../utils/hooks.esm.js';
import { FilterCard } from '../../../FilterCard/FilterCard.esm.js';
import { Progress } from '@backstage/core-components';
import { combinePredicates } from '../../../../utils/types.esm.js';
import { createLevelPredicate, createApplicableRulePredicate, createExemptRulePredicate, createNotEvaluatedRulePredicate, groupAndSystemFilters, toPredicateFilters } from './ScorecardFilterDialogUtils.esm.js';

const useStyles = makeStyles(() => ({
  dialogContent: {
    padding: 0
  }
}));
const ScorecardFilterDialog = ({
  handleClose,
  isOpen,
  ladder,
  scorecard,
  setFilter
}) => {
  const classes = useStyles();
  const { checkedFilters, oneOf } = useFilter();
  const ruleFilterDefinitions = useMemo(() => {
    return mapValues(
      mapByString(scorecard.rules, (rule) => rule.id.toString()),
      (rule) => ({
        display: ruleName(rule),
        value: rule.expression,
        id: rule.id.toString()
      })
    );
  }, [scorecard.rules]);
  const levelDefinitions = useMemo(() => {
    if (!ladder || !ladder.levels) {
      return {};
    }
    const sortedLevels = ladder.levels.sort(
      (level1, level2) => level2.rank - level1.rank
    );
    return {
      "No level": {
        display: "No level",
        value: "",
        id: ""
      },
      ...mapValues(
        mapByString(sortedLevels, (level) => level.id.toString()),
        (level) => ({
          display: level.name,
          value: level.id.toString(),
          id: level.id.toString()
        })
      )
    };
  }, [ladder]);
  const { filterGroups, loading } = useFilters(
    (score) => score.componentRef,
    {
      baseFilters: groupAndSystemFilters
    }
  );
  const filtersDefinition = [
    {
      name: "Levels",
      oneOfDisabled: true,
      filters: levelDefinitions,
      generatePredicate: (level) => createLevelPredicate(level)
    },
    {
      name: "Failing rules",
      filters: ruleFilterDefinitions,
      generatePredicate: (failingRule) => createApplicableRulePredicate(false, failingRule)
    },
    {
      name: "Passing rules",
      filters: ruleFilterDefinitions,
      generatePredicate: (passingRule) => createApplicableRulePredicate(true, passingRule)
    },
    {
      name: "Exempt rules",
      filters: ruleFilterDefinitions,
      generatePredicate: (exemptRule) => createExemptRulePredicate(exemptRule)
    },
    {
      name: "Rules not yet evaluated",
      filters: ruleFilterDefinitions,
      generatePredicate: (notEvaluatedRule) => createNotEvaluatedRulePredicate(notEvaluatedRule)
    },
    ...filterGroups != null ? filterGroups : []
  ];
  const handleSaveFilters = () => {
    const allFilters = [];
    filtersDefinition.forEach((filterDefinition, idx) => {
      var _a;
      const predicateFilters = toPredicateFilters(
        checkedFilters,
        filterDefinition.name
      );
      const filterOneOf = (_a = oneOf[filterDefinition.name]) != null ? _a : true;
      allFilters[idx] = (score) => {
        const results = Object.keys(predicateFilters).filter((id) => predicateFilters[id]).map(
          (id) => filterDefinition.generatePredicate(
            filterDefinition.filters[id].value
          )(score)
        );
        if (results.length === 0) {
          return true;
        }
        return filterOneOf ? results.some(Boolean) : results.every(Boolean);
      };
    });
    setFilter(combinePredicates(Object.values(allFilters)));
    handleClose();
  };
  return /* @__PURE__ */ React__default.createElement(Dialog, { open: isOpen, onClose: handleClose }, /* @__PURE__ */ React__default.createElement(DialogContent, { className: classes.dialogContent }, loading || filterGroups === void 0 ? /* @__PURE__ */ React__default.createElement(Progress, null) : /* @__PURE__ */ React__default.createElement(
    FilterCard,
    {
      filterDefinitions: filtersDefinition,
      title: "Filter Scorecard"
    }
  )), /* @__PURE__ */ React__default.createElement(DialogActions, null, /* @__PURE__ */ React__default.createElement(
    Button,
    {
      onClick: handleSaveFilters,
      color: "primary",
      "aria-label": "Apply filters"
    },
    "Apply filters"
  )));
};
const ScorecardFilterDialogWrapper = (props) => {
  return /* @__PURE__ */ React__default.createElement(FilterProvider, null, /* @__PURE__ */ React__default.createElement(ScorecardFilterDialog, { ...props }));
};

export { ScorecardFilterDialog, ScorecardFilterDialogWrapper };
//# sourceMappingURL=ScorecardFilterDialog.esm.js.map
