import React__default, { useMemo } from 'react';
import { Button, Grid, FormControl, TextField, InputAdornment } from '@material-ui/core';
import { useAsync } from 'react-use';
import { Progress, WarningPanel, EmptyState, Content, ContentHeader, ItemCardGrid } from '@backstage/core-components';
import { useApi } from '@backstage/core-plugin-api';
import { cortexApiRef } from '../../../api/CortexClient.esm.js';
import { ScorecardCard } from '../ScorecardCard/ScorecardCard.esm.js';
import { useInput, useScorecardCompareFn, useDropdown } from '../../../utils/hooks.esm.js';
import { hasText } from '../../../utils/SearchUtils.esm.js';
import SearchIcon from '@material-ui/icons/Search';
import { isNil, isEmpty, isUndefined } from 'lodash';
import { SortDropdown } from '../../Common/SortDropdown.esm.js';

const defaultSortMethods = {
  "Name \u2191": (a, b) => a.name.localeCompare(b.name),
  "Name \u2193": (a, b) => b.name.localeCompare(a.name)
};
const hasFilter = (filter, query) => {
  var _a;
  if (isNil(filter) || !("entityGroupFilter" in filter)) {
    return false;
  }
  return (_a = filter.entityGroupFilter) == null ? void 0 : _a.entityGroups.some(
    (group) => group.toLowerCase().includes(query.toLowerCase())
  );
};
const ScorecardList = () => {
  var _a;
  const cortexApi = useApi(cortexApiRef);
  const [searchQuery, setSearchQuery] = useInput();
  const {
    value: scorecards,
    loading: loadingScorecards,
    error: scorecardsError
  } = useAsync(async () => {
    return await cortexApi.getScorecards();
  }, []);
  const {
    compareFn: scorecardCompareFn,
    loading: loadingScorecardCompareFn,
    error: scorecardCompareFnError
  } = useScorecardCompareFn();
  const customScoresSortMethods = useMemo(() => {
    if (isNil(scorecardCompareFn)) {
      return {};
    }
    return {
      "Custom \u2191": (a, b) => scorecardCompareFn(a, b),
      "Custom \u2193": (a, b) => -1 * scorecardCompareFn(a, b)
    };
  }, [scorecardCompareFn]);
  const scorecardScoresSortMethods = useMemo(() => {
    return {
      ...customScoresSortMethods,
      ...defaultSortMethods
    };
  }, [customScoresSortMethods]);
  const [sortBy, setSortBy] = useDropdown(
    isEmpty(customScoresSortMethods) ? "Name \u2191" : "Custom \u2191",
    [isEmpty(customScoresSortMethods)]
  );
  const scorecardsToDisplay = useMemo(() => {
    const scorecardsToDisplay2 = scorecards == null ? void 0 : scorecards.filter((scorecard) => {
      if (isNil(searchQuery) || isEmpty(searchQuery)) {
        return true;
      }
      return hasText(scorecard, "name", searchQuery) || hasText(scorecard, "description", searchQuery) || hasText(scorecard, "filterQuery", searchQuery) || !isNil(scorecard.filter) && hasFilter(scorecard.filter, searchQuery);
    });
    if (sortBy) {
      scorecardsToDisplay2 == null ? void 0 : scorecardsToDisplay2.sort(scorecardScoresSortMethods[sortBy]);
    }
    return scorecardsToDisplay2;
  }, [scorecards, scorecardScoresSortMethods, searchQuery, sortBy]);
  if (loadingScorecards || loadingScorecardCompareFn || isUndefined(scorecardsToDisplay)) {
    return /* @__PURE__ */ React__default.createElement(Progress, null);
  }
  if (scorecardsError || scorecardCompareFnError) {
    return /* @__PURE__ */ React__default.createElement(WarningPanel, { severity: "error", title: "Could not load Scorecards." }, (_a = scorecardsError == null ? void 0 : scorecardsError.message) != null ? _a : scorecardCompareFnError == null ? void 0 : scorecardCompareFnError.message);
  }
  if (!(scorecards == null ? void 0 : scorecards.length)) {
    return /* @__PURE__ */ React__default.createElement(
      EmptyState,
      {
        missing: "info",
        title: "No scorecards to display",
        description: "You haven't added any scorecards yet.",
        action: /* @__PURE__ */ React__default.createElement(
          Button,
          {
            variant: "contained",
            color: "primary",
            href: "https://backstage.io/docs/features/software-catalog/descriptor-format#kind-domain"
          },
          "Read more"
        )
      }
    );
  }
  return /* @__PURE__ */ React__default.createElement(Content, null, /* @__PURE__ */ React__default.createElement(ContentHeader, { title: "Scorecards" }), /* @__PURE__ */ React__default.createElement(Grid, { container: true, direction: "column" }, /* @__PURE__ */ React__default.createElement(
    Grid,
    {
      container: true,
      direction: "row",
      lg: 12,
      style: { marginBottom: "20px" }
    },
    /* @__PURE__ */ React__default.createElement(Grid, { item: true, lg: 10 }, /* @__PURE__ */ React__default.createElement(FormControl, { fullWidth: true }, /* @__PURE__ */ React__default.createElement(
      TextField,
      {
        variant: "standard",
        placeholder: "Search by name, description, or filters",
        value: searchQuery,
        onChange: setSearchQuery,
        InputProps: {
          startAdornment: /* @__PURE__ */ React__default.createElement(InputAdornment, { position: "start" }, /* @__PURE__ */ React__default.createElement(SearchIcon, null))
        }
      }
    ))),
    /* @__PURE__ */ React__default.createElement(Grid, { item: true, lg: 2 }, /* @__PURE__ */ React__default.createElement(
      SortDropdown,
      {
        selected: sortBy,
        items: Object.keys(scorecardScoresSortMethods),
        select: setSortBy
      }
    ))
  ), /* @__PURE__ */ React__default.createElement(Grid, { item: true, lg: 12 }, isEmpty(scorecardsToDisplay) && !isNil(searchQuery) && /* @__PURE__ */ React__default.createElement(
    EmptyState,
    {
      title: "No Scorecards matching search query",
      missing: "data"
    }
  ), /* @__PURE__ */ React__default.createElement(ItemCardGrid, null, scorecardsToDisplay.map((scorecard) => /* @__PURE__ */ React__default.createElement(ScorecardCard, { key: scorecard.id, scorecard }))))));
};

export { ScorecardList, hasFilter };
//# sourceMappingURL=ScorecardList.esm.js.map
