import React__default, { useState, useCallback, useEffect } from 'react';
import SyncIcon from '@material-ui/icons/Sync';
import CancelIcon from '@material-ui/icons/Cancel';
import { Grid, Typography, IconButton, CircularProgress } from '@material-ui/core';
import { WarningPanel, InfoCard, Link } from '@backstage/core-components';
import { useApi, configApiRef } from '@backstage/core-plugin-api';
import { cortexApiRef } from '../../api/CortexClient.esm.js';
import { catalogApiRef } from '@backstage/plugin-catalog-react';
import { extensionApiRef } from '../../api/ExtensionApi.esm.js';
import PollingLinearGauge from '../Common/PollingLinearGauge.esm.js';
import moment from 'moment';
import { applyCustomMappings } from '../../utils/ComponentUtils.esm.js';

const SyncButton = ({
  isSyncing,
  submitSyncTask
}) => {
  return /* @__PURE__ */ React__default.createElement(
    IconButton,
    {
      onClick: () => submitSyncTask(),
      disabled: isSyncing,
      "aria-busy": isSyncing,
      "aria-label": "Sync Entities"
    },
    isSyncing ? /* @__PURE__ */ React__default.createElement(CircularProgress, null) : /* @__PURE__ */ React__default.createElement(SyncIcon, null)
  );
};
const CancelSyncButton = ({ cancelSync }) => {
  const [isCanceling, setIsCanceling] = useState(false);
  return /* @__PURE__ */ React__default.createElement(
    IconButton,
    {
      onClick: () => {
        setIsCanceling(true);
        cancelSync().finally(() => setIsCanceling(false));
      },
      "aria-busy": isCanceling,
      "aria-label": "Cancel entity sync"
    },
    isCanceling ? /* @__PURE__ */ React__default.createElement(CircularProgress, null) : /* @__PURE__ */ React__default.createElement(CancelIcon, null)
  );
};
const SyncCard = () => {
  const catalogApi = useApi(catalogApiRef);
  const config = useApi(configApiRef);
  const cortexApi = useApi(cortexApiRef);
  const extensionApi = useApi(extensionApiRef);
  const [syncTaskProgressPercentage, setSyncTaskProgressPercentage] = useState(null);
  const [lastSyncedTime, setLastSyncedTime] = useState(null);
  const [isSubmittingTask, setIsSubmittingTask] = useState(false);
  const [cortexSyncError, setCortexSyncError] = useState(
    void 0
  );
  const getBackstageEntities = useCallback(async () => {
    var _a, _b;
    const syncEntityFilter = await ((_a = extensionApi.getSyncEntityFilter) == null ? void 0 : _a.call(extensionApi));
    const { items: entities } = await catalogApi.getEntities(
      (syncEntityFilter == null ? void 0 : syncEntityFilter.kinds) ? { filter: { kind: syncEntityFilter == null ? void 0 : syncEntityFilter.kinds } } : void 0
    );
    const filteredEntities = (syncEntityFilter == null ? void 0 : syncEntityFilter.entityFilter) ? entities.filter(syncEntityFilter == null ? void 0 : syncEntityFilter.entityFilter) : entities;
    const customMappings = await ((_b = extensionApi.getCustomMappings) == null ? void 0 : _b.call(extensionApi));
    const withCustomMappings = customMappings ? filteredEntities.map(
      (entity) => applyCustomMappings(entity, customMappings)
    ) : filteredEntities;
    return withCustomMappings;
  }, [catalogApi, extensionApi]);
  const submitEntitySync = useCallback(async () => {
    var _a, _b;
    setIsSubmittingTask(true);
    const entities = await getBackstageEntities();
    const shouldGzipBody = (_a = config.getOptionalBoolean("cortex.syncWithGzip")) != null ? _a : false;
    const syncChunkSize = config.getOptionalNumber("cortex.syncChunkSize");
    const groupOverrides = await ((_b = extensionApi.getTeamOverrides) == null ? void 0 : _b.call(extensionApi, entities));
    setCortexSyncError(void 0);
    try {
      const progress = await cortexApi.submitEntitySync(
        entities,
        shouldGzipBody,
        groupOverrides,
        syncChunkSize
      );
      setSyncTaskProgressPercentage(progress.percentage);
    } catch (e) {
      setCortexSyncError(e.message);
    }
    setIsSubmittingTask(false);
  }, [getBackstageEntities, config, cortexApi, extensionApi]);
  const cancelEntitySync = useCallback(async () => {
    await cortexApi.cancelEntitySync();
    setSyncTaskProgressPercentage(null);
    const lastSyncedTime2 = (await cortexApi.getLastEntitySyncTime()).lastSynced;
    setLastSyncedTime(lastSyncedTime2);
  }, [cortexApi]);
  const updateEntitySyncProgress = useCallback(async () => {
    const currentProgress = await cortexApi.getEntitySyncProgress();
    if (currentProgress.percentage === null) {
      setLastSyncedTime((await cortexApi.getLastEntitySyncTime()).lastSynced);
    }
    setSyncTaskProgressPercentage(currentProgress.percentage);
  }, [cortexApi]);
  useEffect(() => {
    updateEntitySyncProgress();
  }, [updateEntitySyncProgress]);
  const updateLastEntitySyncTime = useCallback(async () => {
    setLastSyncedTime((await cortexApi.getLastEntitySyncTime()).lastSynced);
  }, [cortexApi]);
  useEffect(() => {
    updateLastEntitySyncTime();
  }, [updateLastEntitySyncTime]);
  return /* @__PURE__ */ React__default.createElement(React__default.Fragment, null, cortexSyncError !== void 0 && /* @__PURE__ */ React__default.createElement(WarningPanel, { severity: "error", title: cortexSyncError }, "There is already a sync in progress. Try again in 10 minutes."), /* @__PURE__ */ React__default.createElement(
    InfoCard,
    {
      title: "Sync entities",
      action: cortexSyncError === void 0 && /* @__PURE__ */ React__default.createElement(
        SyncButton,
        {
          isSyncing: syncTaskProgressPercentage !== null || isSubmittingTask,
          submitSyncTask: submitEntitySync
        }
      )
    },
    syncTaskProgressPercentage !== null && /* @__PURE__ */ React__default.createElement(
      Grid,
      {
        container: true,
        alignItems: "center",
        direction: "row",
        justifyContent: "center",
        style: { marginBottom: "2px" }
      },
      /* @__PURE__ */ React__default.createElement(Grid, { item: true, lg: 10, "data-testid": `PollingLinearGauge-entity-sync` }, /* @__PURE__ */ React__default.createElement(
        PollingLinearGauge,
        {
          done: false,
          poll: updateEntitySyncProgress,
          value: syncTaskProgressPercentage
        }
      )),
      /* @__PURE__ */ React__default.createElement(Grid, { item: true, lg: 2 }, /* @__PURE__ */ React__default.createElement(CancelSyncButton, { cancelSync: cancelEntitySync }))
    ),
    lastSyncedTime !== null ? /* @__PURE__ */ React__default.createElement(Typography, null, /* @__PURE__ */ React__default.createElement("i", null, "Last synced on", " ", moment.utc(lastSyncedTime).local().format("MMM Do YYYY, h:mm:ss a"))) : /* @__PURE__ */ React__default.createElement(Typography, null, /* @__PURE__ */ React__default.createElement("i", null, "Entities have never been synced before.")),
    /* @__PURE__ */ React__default.createElement(Typography, null, "Manually sync your Backstage entities with Cortex.", /* @__PURE__ */ React__default.createElement("br", null), "You can also set this up to automatically sync with our", " ", /* @__PURE__ */ React__default.createElement(Link, { to: "https://www.npmjs.com/package/@cortexapps/backstage-backend-plugin" }, "backend plugin"), ".")
  ));
};

export { SyncCard };
//# sourceMappingURL=SyncCard.esm.js.map
