import { isNil } from 'lodash';
import { RuleOutcomeType, ruleName } from '../api/types.esm.js';
import { isNotNullOrEmpty } from './strings.esm.js';

const isApplicableRuleOutcome = (rule) => (rule == null ? void 0 : rule.type) === RuleOutcomeType.APPLICABLE;
const isNotApplicableRuleOutcome = (rule) => (rule == null ? void 0 : rule.type) === RuleOutcomeType.NOT_APPLICABLE;
const isNotEvaluatedRuleOutcome = (rule) => (rule == null ? void 0 : rule.type) === RuleOutcomeType.NOT_EVALUATED;
const getApplicableRules = (rules) => {
  return rules.filter(isApplicableRuleOutcome);
};
function isRuleOutcomePassing(ruleOutcome) {
  return isApplicableRuleOutcome(ruleOutcome) && ruleOutcome.score > 0;
}
function isRuleOutcomeFailing(ruleOutcome) {
  return isApplicableRuleOutcome(ruleOutcome) && ruleOutcome.score === 0;
}
const isRuleFailing = (rule) => {
  var _a;
  return !isNil(rule.score) && ((_a = rule.score) != null ? _a : 0) <= 0;
};
function filterPassingRuleOutcomes(ruleOutcomes) {
  return ruleOutcomes.filter(isRuleOutcomePassing);
}
function filterFailingRuleOutcomes(ruleOutcomes) {
  return ruleOutcomes.filter(isRuleOutcomeFailing);
}
function filterNotApplicableRuleOutcomes(ruleOutcomes) {
  return ruleOutcomes.filter(
    (ruleOutcome) => ruleOutcome.type === "NOT_APPLICABLE"
  );
}
function filterNotEvaluatedRuleOutcomes(ruleOutcomes) {
  return ruleOutcomes.filter(
    (ruleOutcome) => ruleOutcome.type === "NOT_EVALUATED"
  );
}
const getRuleTitle = (rule) => {
  return isNotNullOrEmpty(rule.title) ? rule.title : rule.expression;
};
function chainedComparator(...comparators) {
  return (a, b) => {
    let order = 0;
    let i = 0;
    while (!order && comparators[i]) {
      order = comparators[i++](a, b);
    }
    return order;
  };
}
function sortRules(rules) {
  return [...rules].sort(
    chainedComparator(
      (a, b) => {
        var _a, _b;
        return ((_a = b == null ? void 0 : b.score) != null ? _a : 0) - ((_b = a == null ? void 0 : a.score) != null ? _b : 0);
      },
      (a, b) => {
        var _a, _b;
        return ((_a = b == null ? void 0 : b.weight) != null ? _a : 0) - ((_b = a == null ? void 0 : a.weight) != null ? _b : 0);
      },
      (a, b) => ruleName(a).localeCompare(ruleName(b))
    )
  );
}

export { chainedComparator, filterFailingRuleOutcomes, filterNotApplicableRuleOutcomes, filterNotEvaluatedRuleOutcomes, filterPassingRuleOutcomes, getApplicableRules, getRuleTitle, isApplicableRuleOutcome, isNotApplicableRuleOutcome, isNotEvaluatedRuleOutcome, isRuleFailing, isRuleOutcomeFailing, isRuleOutcomePassing, sortRules };
//# sourceMappingURL=ScorecardRules.esm.js.map
