import { useState, useCallback, useEffect, useMemo } from 'react';
import { nullsToUndefined, stringifyAnyEntityRef } from './types.esm.js';
import { useAsync } from 'react-use';
import { useApi, configApiRef } from '@backstage/core-plugin-api';
import { catalogApiRef } from '@backstage/plugin-catalog-react';
import { mapValues, mapByString } from './collections.esm.js';
import '@backstage/catalog-model';
import { keyBy, isNil } from 'lodash';
import { cortexApiRef } from '../api/CortexClient.esm.js';
import { extensionApiRef } from '../api/ExtensionApi.esm.js';

function useInput(initialValue = void 0) {
  const [value, setValue] = useState(initialValue);
  const onChange = useCallback(
    (event) => {
      setValue(
        event.target.value === "" ? void 0 : event.target.value
      );
    },
    [setValue]
  );
  return [value, onChange];
}
function useDropdown(initialValue, deps = []) {
  const [value, setValue] = useState(initialValue);
  useEffect(() => {
    setValue(initialValue);
  }, [initialValue, ...deps]);
  const onChange = useCallback(
    (event) => {
      setValue(
        event.target.value === "" ? void 0 : event.target.value
      );
    },
    [setValue]
  );
  return [value, onChange];
}
function useCortexApi(f, deps = []) {
  const cortexApi = useApi(cortexApiRef);
  return useAsync(async () => {
    return await f(cortexApi);
  }, deps);
}
function useCortexApiMemoized(f, deps = []) {
  const cortexApi = useApi(cortexApiRef);
  return useAsyncMemoized(async () => {
    return await f(cortexApi);
  }, deps);
}
function useAsyncMemoized(f, deps) {
  const [cache, setCache] = useState({});
  const cacheKey = useMemo(() => JSON.stringify(deps), [deps]);
  return useAsync(async () => {
    const cachedResult = cache[cacheKey];
    if (!isNil(cachedResult)) {
      return cachedResult;
    }
    const result = await f();
    setCache((prevCache) => ({
      ...prevCache,
      [cacheKey]: result
    }));
    return result;
  }, [cacheKey]);
}
const useServiceGroups = () => {
  const { value, loading } = useCortexApi((api) => api.getServiceGroups());
  return useMemo(
    () => loading || !(value == null ? void 0 : value.serviceGroups) ? [] : value.serviceGroups,
    [loading, value]
  );
};
function useFilters(entityRef, options) {
  var _a;
  const catalogApi = useApi(catalogApiRef);
  const extensionApi = useApi(extensionApiRef);
  const { loading, error, value } = useAsync(async () => {
    var _a2, _b;
    return Promise.all([
      catalogApi.getEntities({
        filter: {
          kind: "component"
        }
      }).then((response) => response.items),
      (_b = (_a2 = extensionApi.getAdditionalFilters) == null ? void 0 : _a2.call(extensionApi)) != null ? _b : []
    ]);
  }, (_a = options == null ? void 0 : options.deps) != null ? _a : []);
  const [components, filters] = value != null ? value : [void 0, void 0];
  const allFilterGroups = useMemo(() => {
    var _a2;
    return ((_a2 = options == null ? void 0 : options.baseFilters) != null ? _a2 : []).concat(filters != null ? filters : []);
  }, [options == null ? void 0 : options.baseFilters, filters]);
  const groupPropertyLookup = useMemo(() => {
    return components === void 0 ? void 0 : allFilterGroups.map((filterGroup) => {
      return components.reduce((mapSoFar, nextComponent) => {
        var _a2;
        (_a2 = filterGroup.groupProperty(nextComponent)) == null ? void 0 : _a2.forEach((groupProperty) => {
          if (mapSoFar[groupProperty] === void 0) {
            mapSoFar[groupProperty] = /* @__PURE__ */ new Set();
          }
          mapSoFar[groupProperty].add(
            stringifyAnyEntityRef(nextComponent)
          );
        });
        return mapSoFar;
      }, {});
    });
  }, [allFilterGroups, components]);
  const filterGroups = useMemo(() => {
    if (components === void 0 || groupPropertyLookup === void 0) {
      return void 0;
    }
    return allFilterGroups.map((filterGroup, index) => {
      return {
        name: filterGroup.name,
        filters: mapValues(
          mapByString(
            components.flatMap(
              (component) => {
                var _a2;
                return (_a2 = filterGroup.groupProperty(component)) != null ? _a2 : [];
              }
            ),
            (groupProperty) => groupProperty
          ),
          (groupProperty) => {
            return {
              display: filterGroup.formatProperty ? filterGroup.formatProperty(groupProperty) : groupProperty,
              value: groupProperty,
              id: groupProperty
            };
          }
        ),
        generatePredicate: (groupProperty) => {
          return (t) => {
            var _a2;
            return Boolean(
              (_a2 = groupPropertyLookup[index][groupProperty]) == null ? void 0 : _a2.has(
                stringifyAnyEntityRef(entityRef(t))
              )
            );
          };
        }
      };
    });
  }, [allFilterGroups, components, groupPropertyLookup, entityRef]);
  const serviceGroups = useServiceGroups();
  const filterGroupsWithServiceGroups = useMemo(() => {
    if ((filterGroups == null ? void 0 : filterGroups.length) && serviceGroups.length) {
      return filterGroups.map(
        (filterGroup) => filterGroup.name === "Groups" ? {
          ...filterGroup,
          filters: {
            ...filterGroup.filters,
            ...Object.fromEntries(
              serviceGroups.map((serviceGroup) => {
                const id = `group:service-group/${serviceGroup}`;
                return [id, { display: serviceGroup, id, value: id }];
              })
            )
          },
          generatePredicate: (groupProperty) => {
            const originalPredicate = filterGroup.generatePredicate(groupProperty);
            return (t) => {
              if (originalPredicate(t)) {
                return true;
              } else {
                const tags = t == null ? void 0 : t.tags;
                const serviceGroup = groupProperty.slice(20);
                return Boolean(tags == null ? void 0 : tags.includes(serviceGroup));
              }
            };
          }
        } : filterGroup
      );
    }
    return filterGroups;
  }, [filterGroups, serviceGroups]);
  return { loading, error, filterGroups: filterGroupsWithServiceGroups };
}
function useCortexFrontendUrl() {
  var _a;
  const config = useApi(configApiRef);
  return (_a = config.getOptionalString("cortex.frontendBaseUrl")) != null ? _a : "https://app.getcortexapp.com";
}
function useUiExtensions() {
  const extensionApi = useApi(extensionApiRef);
  const { value: uiExtensions, ...rest } = useAsync(async () => {
    return isNil(extensionApi.getUiExtensions) ? void 0 : await extensionApi.getUiExtensions();
  }, []);
  return {
    uiExtensions,
    ...rest
  };
}
function useInitiativesCustomName() {
  var _a, _b;
  const config = useApi(configApiRef);
  const singular = (_a = config.getOptionalString("cortex.initiativeNameOverride.singular")) != null ? _a : "Initiative";
  const plural = (_b = config.getOptionalString("cortex.initiativeNameOverride.plural")) != null ? _b : `${singular}s`;
  return { singular, plural };
}
function useHideCortexLinks() {
  var _a;
  const config = useApi(configApiRef);
  return (_a = config.getOptionalBoolean("cortex.hideCortexLinks")) != null ? _a : false;
}
const defaultCompareFn = (a, b) => a.name.localeCompare(b.name);
function useScorecardCompareFn() {
  var _a, _b;
  const { uiExtensions, ...rest } = useUiExtensions();
  const customCompareFn = (_b = (_a = uiExtensions == null ? void 0 : uiExtensions.scorecards) == null ? void 0 : _a.sortOrder) == null ? void 0 : _b.compareFn;
  const sanitizedCompareFn = isNil(customCompareFn) ? void 0 : (a, b) => {
    return customCompareFn(nullsToUndefined(a), nullsToUndefined(b));
  };
  return { ...rest, compareFn: sanitizedCompareFn != null ? sanitizedCompareFn : defaultCompareFn };
}
function usePartialScorecardCompareFn() {
  var _a, _b;
  const {
    uiExtensions,
    loading: loadingUiExtensions,
    error: uiExtensionsError
  } = useUiExtensions();
  const {
    value: scorecardsById,
    loading: loadingScorecards,
    error: scorecardsError
  } = useCortexApi(
    async (api) => keyBy(await api.getScorecards(), (scorecard) => scorecard.id)
  );
  const scorecardCompareFn = (_b = (_a = uiExtensions == null ? void 0 : uiExtensions.scorecards) == null ? void 0 : _a.sortOrder) == null ? void 0 : _b.compareFn;
  const compareFn = !isNil(scorecardCompareFn) && !isNil(scorecardsById) ? (a, b) => scorecardCompareFn(
    nullsToUndefined(scorecardsById[a.id]),
    nullsToUndefined(scorecardsById[b.id])
  ) : void 0;
  return {
    loading: loadingUiExtensions || loadingScorecards,
    error: uiExtensionsError != null ? uiExtensionsError : scorecardsError,
    compareFn
  };
}
function useEntitiesByTag() {
  const { value: entities, loading } = useCortexApi(
    (api) => api.getCatalogEntities(),
    []
  );
  const entitiesByTag = useMemo(
    () => !isNil(entities) && !isNil(entities.entities) ? keyBy(Object.values(entities.entities), (entity) => entity.codeTag) : {},
    [entities]
  );
  return { entitiesByTag, loading };
}

export { useCortexApi, useCortexApiMemoized, useCortexFrontendUrl, useDropdown, useEntitiesByTag, useFilters, useHideCortexLinks, useInitiativesCustomName, useInput, usePartialScorecardCompareFn, useScorecardCompareFn, useUiExtensions };
//# sourceMappingURL=hooks.esm.js.map
